@@ -61,14 +61,14 @@ void DomParser::parse_file(const Glib::ustring& filename)
   parse_context();
 }
 
-void DomParser::parse_memory(const Glib::ustring& contents)
+void DomParser::parse_memory_raw(const unsigned char* contents, size_type bytes_count)
 {
   release_underlying(); //Free any existing document.
 
   KeepBlanks k(KeepBlanks::Default);
 
   //The following is based on the implementation of xmlParseFile(), in xmlSAXParseFileWithData():
-  context_ = xmlCreateMemoryParserCtxt(contents.c_str(), contents.bytes());
+  context_ = xmlCreateMemoryParserCtxt((const char*)contents, bytes_count);
 
   if(!context_)
   {
@@ -78,6 +78,11 @@ void DomParser::parse_memory(const Glib::ustring& contents)
   parse_context();
 }
 
+void DomParser::parse_memory(const Glib::ustring& contents)
+{
+  parse_memory_raw((const unsigned char*)contents.c_str(), contents.bytes());
+}
+
 void DomParser::parse_context()
 {
   KeepBlanks k(KeepBlanks::Default);
@@ -86,6 +91,11 @@ void DomParser::parse_context()
   //and the implementation of xmlParseMemory(), in xmlSaxParseMemoryWithData().
   initialize_context();
 
+  if(!context_)
+  {
+    throw internal_error("Context not initialized");
+  }
+
   xmlParseDocument(context_);
 
   check_for_exception();
@@ -98,9 +108,11 @@ void DomParser::parse_context()
 
   if(context_->errNo != 0)
   {
-    release_underlying();
     std::ostringstream o;
     o << "libxml error " << context_->errNo;
+
+    release_underlying();
+
     throw parse_error(o.str());
   }
 
@@ -162,7 +174,9 @@ void DomParser::parse_stream(std::istream& in)
   {
     std::ostringstream o;
     o << "libxml error " << context_->errNo;
+
     release_underlying();
+
     throw parse_error(o.str());
   }
 
